# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import unittest

from h2md import convert


class TestConvert(unittest.TestCase):

    def test_code_block(self):
        html = "Code block<pre><code>print('hello markdown')</code></pre>"
        expected = '\n'.join([
            'Code block',
            '```',
            "print('hello markdown')",
            '```',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_code_block_with_span(self):
        html = "Code block<pre><code><span>print('hello markdown')<span></code></pre>"
        expected = '\n'.join([
            'Code block',
            '```',
            "print('hello markdown')",
            '```',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_code_block_with_newline(self):
        html = "Code block<pre><code><span>print('hello markdown')<span>\n<span>print('hello html')<span></code></pre>"
        expected = '\n'.join([
            'Code block',
            '```',
            "print('hello markdown')",
            "print('hello html')",
            '```',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_code_block_with_lang(self):
        html = '''Code block:<pre><code class="hljs bash">kubectl <span class="hljs-built_in">exec</span> productpage-v1-54b8b9f55-bx2dq -c istio-proxy -- cat /etc/istio/proxy/envoy-rev0.json &gt; envoy-rev0.json
</code></pre>'''
        expected = '\n'.join([
            'Code block:',
            '```bash',
            "kubectl exec productpage-v1-54b8b9f55-bx2dq -c istio-proxy -- cat /etc/istio/proxy/envoy-rev0.json > envoy-rev0.json",
            "```",
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_image(self):
        html = 'Image <img alt="alt" src="src" />'
        expected = 'Image ![alt](src)'
        self.assertEqual(convert(html), expected)

    def test_hyperlinks(self):
        html = 'Hyperlink <a href="href">text</a>'
        expected = 'Hyperlink [text](href)'
        self.assertEqual(convert(html), expected)

    def test_hyperlinks_no_href(self):
        html = 'Hyperlink <a>text</a>'
        expected = 'Hyperlink [text](text)'
        self.assertEqual(convert(html), expected)

    def test_image_no_alt_src(self):
        html = 'Image <img />'
        expected = 'Image ![]()'
        self.assertEqual(convert(html), expected)

    def test_italics(self):
        html = 'Italics <i>i</i> <em>em</em>'
        expected = 'Italics *i* *em*'
        self.assertEqual(convert(html), expected)

    def test_bold(self):
        html = 'Bold <b>b</b> <strong>strong</strong>'
        expected = 'Bold **b** **strong**'
        self.assertEqual(convert(html), expected)

    def test_deleted(self):
        html = 'Deleted <del>del</del>'
        expected = 'Deleted ~~del~~'
        self.assertEqual(convert(html), expected)

    def test_code(self):
        html = 'Code <code>code</code>'
        expected = 'Code `code`'
        self.assertEqual(convert(html), expected)

    def test_nested(self):
        html = 'Nested <code><del><b><i>nested</i></b></del></code>'
        expected = 'Nested `~~***nested***~~`'
        self.assertEqual(convert(html), expected)

    def test_inline_empty(self):
        html = 'Inline empty <code></code><del></del><b></b><i></i>'
        expected = 'Inline empty '
        self.assertEqual(convert(html), expected)

    def test_headers(self):
        html = '\n'.join([
            '<h1>H1</h1>',
            '<h2>H2</h2>',
            '<h3>H3</h3>',
            '<h4>H4</h4>',
            '<h5>H5</h5>',
            '<h6>H6</h6>',
        ])
        expected = '\n'.join([
            '',
            '# H1',
            '',
            '## H2',
            '',
            '### H3',
            '',
            '#### H4',
            '',
            '##### H5',
            '',
            '###### H6',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_horizontal_ruler(self):
        html = 'Horizontal ruler<hr>'
        expected = '\n'.join([
            'Horizontal ruler',
            '',
            '---',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_paragraph(self):
        html = 'Paragraph<p>a</p><p>b</p>'
        expected = '\n'.join([
            'Paragraph',
            '',
            'a',
            '',
            'b',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_quotes(self):
        html = 'Quotes<blockquote>blockquote</blockquote>'
        expected = '\n'.join([
            'Quotes',
            '',
            '> blockquote',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_quotes_nested(self):
        html = '''Nested quotes<blockquote>
        blockquote
        <blockquote>nested</blockquote>
        <blockquote>
            nested
            <blockquote>nested double</blockquote>
        </blockquote>
        </blockquote>'''
        expected = '\n'.join([
            'Nested quotes',
            '',
            '> blockquote',
            '> ',
            '> > nested',
            '> ',
            '> > nested',
            '> > ',
            '> > > nested double',
            '',
            '',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_lists_ordered_empty(self):
        html = 'Ordered list<ol></ol>'
        expected = 'Ordered list'
        self.assertEqual(convert(html), expected)

    def test_lists_ordered(self):
        html = '''Ordered list<ol>
        <li>item 1</li>
        <li>item 2</li>
        </ol>'''
        expected = '\n'.join([
            'Ordered list',
            '',
            '1. item 1',
            '2. item 2',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_lists_unordered_empty(self):
        html = 'Unordered list<ul></ul>'
        expected = 'Unordered list'
        self.assertEqual(convert(html), expected)

    def test_lists_unordered(self):
        html = '''Unordered list<ul>
        <li>item 1</li>
        <li>item 2</li>
        </ul>'''
        expected = '\n'.join([
            'Unordered list',
            '',
            '* item 1',
            '* item 2',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_lists_nested(self):
        html = '''Nested list<ol>
        <li>item 1</li>
        <li>item 2
            <ul>
            <li>item 2.1</li>
            <li>item 2.2
                <ul>
                <li>item 2.2.1</li>
                </ul>
            </li>
            </ul>
        </li>
        </ol>'''
        expected = '\n'.join([
            'Nested list',
            '',
            '1. item 1',
            '2. item 2',
            '',
            '    * item 2.1',
            '    * item 2.2',
            '',
            '        * item 2.2.1',
            '',
            '',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_nested_tags_in_lists(self):
        html = '''Nested tags in lists
        <ul>
            <li>item
            <pre><code>b</code></pre>
            <a href="href">text</a>
            <img alt="alt" src="src" />
            <h1>H1</h1>
            <p>
                <i>i</i> <em>em</em> <b>b</b> <strong>strong</strong>
                <del>del</del> <code>code</code>
            </p>
            <blockquote>
                blockquote
                <blockquote>nested</blockquote>
            </blockquote>
            <ol>
                <li>ordered list item</li>
                <li>
                <p>
                    <i>i</i> <em>em</em> <b>b</b> <strong>strong</strong>
                    <del>del</del> <code>code</code>
                </p>
                </li>
            </ol>
            <ul>
                <li>unordered list item</li>
                <li>
                <p>
                    <i>i</i> <em>em</em> <b>b</b> <strong>strong</strong>
                    <del>del</del> <code>code</code>
                </p>
                </li>
            </ul>
            </li>
        </ul>
        '''
        expected = '\n'.join([
            'Nested tags in lists',
            '',
            '* item',
            '```',
            'b',
            '```',
            '[text](href) ![alt](src)',
            '# H1',
            '',
            '*i* *em* **b** **strong** ~~del~~ `code`',
            '',
            '> blockquote',
            '> ',
            '> > nested',
            '',
            '',
            '    1. ordered list item',
            '    2. *i* *em* **b** **strong** ~~del~~ `code`',
            '',
            '',
            '    * unordered list item',
            '    * *i* *em* **b** **strong** ~~del~~ `code`',
            '',
            '',
            ''
        ])
        self.assertEqual(convert(html), expected)

    def test_other_tags(self):
        html = """
        <center>Pilot Architecture（来自<a href="https://istio.io/docs/concepts/traffic-management/">Isio官网文档</a><sup><a href="#ref01">[1]</a></sup>)</center>
        """
        expected = '<center>Pilot Architecture（来自[Isio官网文档](https://istio.io/docs/concepts/traffic-management/)<sup>[[1]](#ref01)</sup>)</center>'
        self.assertEqual(convert(html), expected)

if __name__ == '__main__':
    unittest.main()



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import argparse
import fileinput

import h2md


def main(argv=None):

  parser = argparse.ArgumentParser()
  parser.add_argument('file', default='-', nargs='?', help='HTML 正文部分转 Markdown')
  args = parser.parse_args(argv)

  html = ''.join([line for line in fileinput.input(args.file)])
  print(h2md.convert(html))


if __name__ == '__main__':
  main()



#!/usr/bin/env python

# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements. See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership. The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import re
from bs4 import NavigableString, BeautifulSoup as bs

inline_tags = ['a', 'img', 'b', 'strong', 'em', 'i', 'code', 'del']
# block_tags = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'hr', 'blockquote', 'pre']

block_map = {
    'normal': {
        'h1': '\n# {}\n',
        'h2': '\n## {}\n',
        'h3': '\n### {}\n',
        'h4': '\n#### {}\n',
        'h5': '\n##### {}\n',
        'h6': '\n###### {}\n',
        'hr': '\n\n---\n',
        'div': '{}'
    },
    'intent': {
        'p': '\n{}{}\n',
        'blockquote': '{}> {}',
        'pre': '\n{}```{}\n{}\n{}```\n'
    }
}

inline_map = {
    'normal': {
        'i': '*{}*',
        'em': '*{}*',
        'b': '**{}**',
        'strong': '**{}**',
        'del': '~~{}~~',
        "code": '`{}`'
    },
    'link': {
        'a': '[{}]({})',
        'img': '![{}]({})'
    }
}

def convert(html):
    soup = bs(html, 'html.parser')
    container = soup.select_one('article .post-container') \
        or soup.select_one('article #content') \
        or soup.select_one('article') \
        or soup.select_one('body') \
        or soup
    return __print_tree(container)

def __print_tree(ele, intent = 0, md = ''):
    """递归遍历DOM，为了开发时间暂时就用递归了
    
    Arguments:
        ele {bs} -- 待解析元素
    
    Keyword Arguments:
        intent {int} -- 缩进值 (default: {0})
        md {str} -- 转换后的文档 (default: {''})
    
    Returns:
        str -- 转换后的文档
    """
    if isinstance(ele, NavigableString):
        md = __transform_text(ele, md)
    elif ele.name == 'img':
        md = __transform_img(ele, md)
    elif ele.name == 'a':
        md = __transform_a(ele, md, intent)
    elif ele.name in inline_map['normal'].keys():
        md = __transform_inline_tags(ele, md, intent)
    elif ele.name == 'pre':
        md =  __transform_pre(ele, md, intent)
    elif ele.name in ('ul', 'ol'):
        md = __transform_list_tags(ele, md, intent)
    elif ele.name in block_map['normal'].keys():
        md = __transform_block_normal_tags(ele, md, intent)
    elif ele.name in block_map['intent'].keys():
        md = __transform_block_intent_tags(ele, md, intent)
    elif ele.name == '[document]':
        md = __transform_soup(ele, md, intent)
    else:
        md = __transform_other_tags(ele, md, intent)

    return md
    
def __transform_text(ele, md):
    text = re.compile(r'[\s]+').sub(' ', ele.string)
    text = text if ele.previous_sibling and ele.previous_sibling.name in inline_tags else text.lstrip()
    text = text if ele.next_sibling and ele.next_sibling.name in inline_tags else text.rstrip()
    md += text

    return md

def __transform_img(ele, md):
    md += inline_map['link']['img'].format(ele.get('alt') or '', ele.get('src') or '')

    return md

def __transform_a(ele, md, intent):
    a_inner = ''
    for child in ele.children:
        a_inner = __print_tree(child, intent, a_inner)
    md += inline_map['link']['a'].format(a_inner, ele.get('href') or ele.get_text(strip=True))

    return md

def __transform_pre(ele, md, intent):
    lang_tag = ele.find(class_='hljs')
    if lang_tag: lang_tag['class'].remove('hljs')
    lang = ''.join(lang_tag['class']) if lang_tag else ''
    md += block_map['intent']['pre'].format(' ' * intent, lang, ele.text.strip().replace('\n', '\n' + ' ' * intent), ' ' * intent)
    
    return md

def __transform_inline_tags(ele, md, intent):
    inline_tag_inner = ''
    for child in ele.children:
        inline_tag_inner = __print_tree(child, intent, inline_tag_inner)
    if inline_tag_inner:
        md += inline_map['normal'][ele.name].format(inline_tag_inner)

    return md

def __transform_block_normal_tags(ele, md, intent):
    block_tag_inner = ''
    for child in ele.children:
        block_tag_inner = __print_tree(child, intent, block_tag_inner)
    md += block_map['normal'][ele.name].format(block_tag_inner)

    return md

def __transform_block_intent_tags(ele, md, intent):
    block_tag_inner = ''
    tpl = block_map['intent'][ele.name]
    prev = ' ' * intent

    if ele.parent.name == 'blockquote':
        prev = ele.parent['data-prev']
        ele['data-prev'] = ele.parent['data-prev'] + '> '
        tpl = ele.parent['data-prev'] + '\n' + tpl + '\n'
    elif ele.name == 'blockquote':
        tpl = '\n' + tpl + '\n'
        ele['data-prev'] = ' ' * intent + '> '

    for child in ele.children:
        block_tag_inner = __print_tree(child, intent, block_tag_inner)
    
    tpl = __fill_newline_if_need(ele, tpl)
    md += tpl.format(prev, block_tag_inner)

    return md

def __transform_other_tags(ele, md, intent):
    other_inner = ''
    for child in ele.children:
        other_inner = __print_tree(child, intent, other_inner)
    
    ele.clear()
    ele.append('{}')
    md += ele.decode().format(other_inner)

    return md

def __transform_list_tags(ele, md, intent):
    list_text = '\n'
    if ele.find_parent(re.compile('[ou]l')): intent += 4
    
    line_head = '* ' if ele.name == 'ul' else '{}. '
    for i, e in enumerate(ele.find_all('li', recursive=False)):
        li_inner = ''
        for child in e.children:
            li_inner = __print_tree(child, intent, li_inner)
        list_text += ' ' * intent + line_head.format(i + 1) + li_inner.lstrip() + '\n'
    
    md += __fill_newline_if_need(ele, list_text) if list_text.strip() != '' else ''

    return md

def __transform_soup(ele, md, intent):
    for child in ele.children:
        md = __print_tree(child, intent, md)
        
    return md

def __fill_newline_if_need(ele, text):
    if ele.next_sibling and ele.next_sibling.name in inline_map['normal'].keys() \
        or isinstance(ele.next_sibling, NavigableString) and ele.next_sibling.string.strip() != '':
        text += '\n'

    if ele.previous_sibling and ele.previous_sibling in inline_map['normal'].keys()\
        or isinstance(ele.previous_sibling, NavigableString) and ele.previous_sibling.string.strip() != '':
        text = '\n' + text
    
    return text